@@ -1,9 +1,9 @@
 import contextlib
 import enum
 import functools
-import operator
 from dataclasses import dataclass, field
 from typing import Any, Callable, Dict, List, Optional, Union, ContextManager
+import importlib
 
 from requests import get, post  # As for API
 import selenium
@@ -17,6 +17,7 @@ from selenium.webdriver.support.ui import WebDriverWait
 
 from . import internal
 
+
 __all__ = [
     "get_browser",
     "Browser",
@@ -29,7 +30,6 @@ __all__ = [
 ]
 
 
-@internal.add_typo_safety
 class BrowserType(enum.Enum):
     """An enumeration of popular, supported browsers for browser automation"""
 
@@ -38,18 +38,22 @@ class BrowserType(enum.Enum):
     FIREFOX = "Firefox"
 
 
-@internal.add_typo_safety
-@contextlib.contextmanager
-def Browser(
-    url: str, start_browser: webdriver.WebDriver = getattr(webdriver, get_browser())
-) -> ContextManager[Browser]:
-    """A context manager for starting a browser automation session"""
-    driver = start_browser()
-    driver.get(url)
-    try:
-        yield BrowserContext(url, driver)
-    finally:
-        driver.close()
+def get_browser(
+    priority=(BrowserType.CHROME, BrowserType.FIREFOX, BrowserType.SAFARI)
+) -> str:
+    def _(browser_name: BrowserType):
+        try:
+            importlib.import_module(browser_name.value, webdriver)().close()
+        except (ModuleNotFoundError, selenium.common.exceptions.WebDriverException):
+            return None
+        else:
+            return browser_name
+
+    found = functools.reduce(lambda x, y: x or y, map(_, priority))
+    if found is None:
+        # TODO: Download geckodriver and use it
+        raise RuntimeError("Could not find any browser")
+    return found
 
 
 @internal.add_typo_safety
@@ -136,33 +140,20 @@ class BrowserContext:
     # Aliases
 
 
-def get_browser(
-    priority=(BrowserType.CHROME, BrowserType.FIREFOX, BrowserType.SAFARI)
-) -> str:
-    def _(browser_name: str):
-        try:
-            getattr(webdriver, browser_name)()
-        except selenium.common.exceptions.WebDriverException:
-            return None
-        else:
-            return browser_name
+@internal.add_typo_safety
+@dataclass
+class Browser(contextlib.AbstractContextManager):
+    url: str
+    start_browser: Optional[webdriver.WebDriver] = None
 
-    found = functools.reduce(operator.or_, map(_, priority))
-    if found is None:
-        # TODO: Download geckodriver and use it
-        raise RuntimeError("Could not find any browser")
-    return found
+    def __post_init__(self) -> None:
+        if self.start_browser is None:
+            self.start_browser = getattr(webdriver, get_browser())
 
+    def __enter__(self) -> BrowserContext:
+        self.driver = self.start_browser()
+        self.driver.get(self.url)
+        return BrowserContext(self.url, self.driver)
 
-#
-# class BrowserType:
-#     def __init__(self, url: str):
-#         self.browser = get_browser()
-#         self.url = url
-#
-#     def __enter__(self):
-#         self.browser.get(self.url)
-#         return self
-#
-#     def __exit__(self, *exc):
-#         self.browser.close()
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.driver.close()
